from typing import List, Tuple

from dataclasses import dataclass, field


@dataclass
class Template:

    name: str
    path_prefix: str = "{}-{}-{}"
    path_patterns: list[str] = field(default_factory=list)


@dataclass
class Manifest:

    name: str
    template: Template
    args: tuple[str] = field(default_factory=tuple)
    path_patterns: list[str] = field(default_factory=list)


@dataclass
class LocalComparisonResult:

    manifest: Manifest
    local_dir: str
    resolved_paths: list[tuple[str, list[str]]] = field(default_factory=list)



# type: ignore[attr-defined]
from typing import Optional

import json
from enum import Enum
from random import choice

import typer
import yaml
from rich.console import Console

from data_manifestor import core, version
from data_manifestor.example import hello


class Color(str, Enum):
    white = "white"
    red = "red"
    cyan = "cyan"
    magenta = "magenta"
    yellow = "yellow"
    green = "green"


app = typer.Typer(
    name="data_manifestor",
    help="Awesome `data_manifestor` is a Python cli/package created with https://github.com/TezRomacH/python-package-template",
    add_completion=False,
)
console = Console()


def version_callback(print_version: bool) -> None:
    """Print the version of the package."""
    if print_version:
        console.print(f"[yellow]data_manifestor[/] version: [bold blue]{version}[/]")
        raise typer.Exit()


@app.command(name="")
def main(
    name: str = typer.Option(..., help="Person to greet."),
    color: Optional[Color] = typer.Option(
        None,
        "-c",
        "--color",
        "--colour",
        case_sensitive=False,
        help="Color for print. If not specified then choice will be random.",
    ),
    print_version: bool = typer.Option(
        None,
        "-v",
        "--version",
        callback=version_callback,
        is_eager=True,
        help="Prints the version of the data_manifestor package.",
    ),
) -> None:
    """Print a greeting with a giving name."""
    if color is None:
        color = choice(list(Color))

    greeting: str = hello(name)
    console.print(f"[bold {color}]{greeting}[/]")


@app.command()
def find_files(
    directory: str,
    template_path: str,
    lims_id: str,
    subject_id: str,
    date_str: str,
    output_path: str,
    fail_if_missing: bool = typer.Option(
        None,
        "-f",
        "--fail",
        help="Raise an exception if files are missing.",
    ),
):
    with open(template_path) as f:
        template = yaml.load(f, Loader=yaml.Loader)

    results = core.compare_manifest_to_local(
        template,
        directory,
        lims_id,
        subject_id,
        date_str,
    )

    filepaths = []
    for (
        pattern,
        paths,
    ) in results.resolved_paths:
        if fail_if_missing and not len(paths) > 0:
            raise Exception("Path not found for: %s" % pattern)
        filepaths.extend(paths)

    with open(output_path, "w") as f:
        json.dump(filepaths, f)


if __name__ == "__main__":
    app()



from marshmallow import (
    Schema,
    ValidationError,
    fields,
    post_load,
    validate,
    validates_schema,
)

from .. import dataclasses


class Dataclass(Schema):

    __model__ = None

    @post_load
    def make_dataclass(self, data: dict, **kwargs):
        return self.__model__(**data)


class Template(Dataclass):

    __model__ = dataclasses.Template

    name = fields.String()
    path_prefix = fields.String(dump_default="{}-{}-{}")
    path_patterns = fields.List(fields.String())



from ..dataclasses import Template
from .schemas import Template as TemplateSchema

template_schema = TemplateSchema()


def dict_to_template(data: dict) -> Template:
    return template_schema.load(data)



from typing import List, Tuple

import os
from glob import glob


class PathException(Exception):
    """Pathing related Exception for more specific error handling."""


def resolve_paths(
    parent_dir: str,
    path_pattern: str,
) -> tuple[str, list[str]]:
    path = os.path.join(parent_dir, path_pattern)
    return (
        path,
        glob(path),
    )


def generate_path_pattern(
    path_prefix: str,
    path_pattern: str,
    *args,
) -> str:
    full_pattern = path_prefix + path_pattern

    try:
        return full_pattern.format(*args)
    except IndexError:
        raise PathException(
            "Incompatible path_pattern and args combination. path_pattern=%s, args=%s",
            full_pattern,
            args,
        )



import os
from glob import glob

from .dataclasses import LocalComparisonResult, Manifest
from .paths import generate_path_pattern, resolve_paths
from .transformers import dict_to_template


def generate_manifest(template_dict: dict, *args: str) -> Manifest:
    template = dict_to_template(template_dict)
    return Manifest(
        name=template.name,
        template=template,
        args=args,
        path_patterns=[
            generate_path_pattern(template.path_prefix, path_pattern, *args)
            for path_pattern in template.path_patterns
        ],
    )


def compare_manifest_to_local(
    template_dict: dict, local_dir: str, *args: str
) -> LocalComparisonResult:
    manifest = generate_manifest(template_dict, *args)
    return LocalComparisonResult(
        local_dir=local_dir,
        manifest=manifest,
        resolved_paths=[
            resolve_paths(local_dir, path_pattern)
            for path_pattern in manifest.path_patterns
        ],
    )
